-------------------------------------------------------------- 
--  Subject : Hilbert&Serpinski curves
--  Copyright (c) 1999 by A.Baranovsky
--------------------------------------------------------------

uses Import

constant
  Square = 512
  Count  = 4
end

class Hilbert
  function Init( iForm )
  function A( K )
  function B( K )
  function C( K )
  function D( K )
  function Line( Dir, L )
  function Draw()
end

implementation Hilbert

  local I, U, X0, Y0, Form

  function Init( iForm )
    object &Form : TForm
    Form == iForm

    R := Form.GetRect()
    writeln( R )

    I  := 0
    U  := Square
    X0 := 250
    Y0 := 270
  return 0

  function A( K )
    if K > 0 then
      D( K - 1 )
      Line( 4, U )
      A( K - 1 )
      Line( 6, U )
      A( K - 1 )
      Line( 0, U )
      B( K - 1 )
    end;
  return 0

  function B( K )
    if K > 0 then
      C( K - 1 )
      Line( 2, U )
      B( K - 1 )
      Line( 0, U )
      B( K - 1 )
      Line( 6, U )
      A( K - 1 )
    end;
  return 0

  function C( K )
    if K > 0 then
      B( K - 1 )
      Line( 0, U )
      C( K - 1 )
      Line( 2, U )
      C( K - 1 )
      Line( 4, U )
      D( K - 1 )
    end;
  return 0

  function D( K )
    if K > 0 then
      A( K - 1 )
      Line( 6, U )
      D( K - 1 )
      Line( 4, U )
      D( K - 1 )
      Line( 2, U )
      C( K - 1 )
    end;
  return 0

  function Line( Dir, L )
    X := Form.GetPenPosX()
    Y := Form.GetPenPosY()
    if      Dir = 0 then
      X := X + L
    elsif Dir = 1 then
      X := X + L
      Y := Y + L
    elsif Dir = 2 then
      Y := Y + L
    elsif Dir = 3 then
      X := X - L
      Y := Y + L
    elsif Dir = 4 then
      X := X - L
    elsif Dir = 5 then
      X := X - L
      Y := Y - L
    elsif Dir = 6 then
      Y := Y - L
    elsif Dir = 7 then
      X := X + L
      Y := Y - L
    end
    Form.LineTo( X, Y )
    Form.MoveTo( X, Y )
  return 0

  function Draw()
    repeat
      I  := I + 1
      U  := U / 2
      X0 := X0 + ( U / 2 )
      Y0 := Y0 + ( U / 2 )
      Form.MoveTo( X0, Y0 )
      A( I )
    until I = Count
  return 0
end

class Serpinski
  function Init( iForm )
  function A( K )
  function B( K )
  function C( K )
  function D( K )
  function Line( Dir, L )
  function Draw()
end

implementation Serpinski

  local I, H, X0, Y0, Form

  function Init( iForm )
    object &Form : TForm
    Form == iForm
    I  := 0
    H  := Square / 4
    X0 := 300
    Y0 := 400
  return 0

  function A( K )
    if K > 0 then
      A( K - 1 )
      Line( 7, H )
      B( K - 1 )
      Line( 0, 2*H )
      D( K - 1 )
      Line( 1, H )
      A( K - 1 )
    end;
  return 0

  function B( K )
    if K > 0 then
      B( K - 1 )
      Line( 5, H )
      C( K - 1 )
      Line( 6, 2*H )
      A( K - 1 )
      Line( 7, H )
      B( K - 1 )
    end;
  return 0

  function C( K )
    if K > 0 then
      C( K - 1 )
      Line( 3, H )
      D( K - 1 )
      Line( 4, 2*H )
      B( K - 1 )
      Line( 5, H )
      C( K - 1 )
    end;
  return 0

  function D( K )
    if K > 0 then
      D( K - 1 )
      Line( 1, H )
      A( K - 1 )
      Line( 2, 2*H )
      C( K - 1 )
      Line( 3, H )
      D( K - 1 )
    end;
  return 0

  function Line( Dir, L )
    X := Form.GetPenPosX()
    Y := Form.GetPenPosY()
    if      Dir = 0 then
      X := X + L
    elsif Dir = 1 then
      X := X + L
      Y := Y + L
    elsif Dir = 2 then
      Y := Y + L
    elsif Dir = 3 then
      X := X - L
      Y := Y + L
    elsif Dir = 4 then
      X := X - L
    elsif Dir = 5 then
      X := X - L
      Y := Y - L
    elsif Dir = 6 then
      Y := Y - L
    elsif Dir = 7 then
      X := X + L
      Y := Y - L
    end
    Form.LineTo( X, Y )
    Form.MoveTo( X, Y )
  return 0

  function Draw()
    repeat
      I  := I + 1
      X0 := X0 - H
      H  := H / 2
      Y0 := Y0 + H
      Form.MoveTo( X0, Y0 );
      A( I )
      Line( 7, H )
      B( I )
      Line( 5, H )
      C( I )
      Line( 3, H )
      D( I )
      Line( 1, H )
    until I = Count
  return 0
end

class THilbertForm : TForm
  function Paint()
end

implementation THilbertForm

  function Paint()

    TForm.Paint()
--    writeln( 'Paint THilbertForm' )

    object G  : Hilbert
    G.Init( Self() )

    G.Draw()
  return 0

end

class TSerpinskiForm : TForm
  function Paint()
end

implementation TSerpinskiForm

  function Paint()

    TForm.Paint()
--    writeln( 'Paint TSerpinskiForm' )  

    object S  : Serpinski
    S.Init( Self() )

    S.Draw()
  return 0

end

--------- Main program -----------------------------------------

  object Form1 : THilbertForm
  object Form2 : TSerpinskiForm

  Form1.Create( -1, 'Hilbert curves', 0, 0, 600, 450 )
  Form1.Show()

  Form2.Create( -1, 'Serpinski curves', 100, 100, 300, 200 )
  Form2.Show()  
return

